@@ -1,11 +1,12 @@
 package Screensaver::Any;
 
-our $DATE = '2017-03-31'; # DATE
-our $VERSION = '0.002'; # VERSION
+our $DATE = '2018-02-28'; # DATE
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
+use Log::ger;
 
 use Exporter::Rinci qw(import);
 use File::Which qw(which);
@@ -61,10 +62,21 @@ sub detect_screensaver {
 
   KDE:
     {
-        last unless which("qdbus");
-        system({capture_stdout=>\my $dummy_out, capture_stderr=>\my $dummy_err},
+        log_trace "Checking qdbus program ...";
+        unless (which("qdbus")) {
+            log_trace "qdbus doesn't exist";
+            last;
+        }
+        log_trace "qdbus exists";
+        system({log=>1,
+                capture_stdout=>\my $dummy_out, capture_stderr=>\my $dummy_err},
                "qdbus", "org.kde.screensaver");
-        last if $?;
+        if ($?) {
+            log_trace "Couldn't check org.kde.screensaver dbus service";
+            last;
+        }
+        log_trace "org.kde.screensaver dbus service exists";
+        log_trace "Concluding screensaver is kde";
         return "kde";
     }
 
@@ -74,19 +86,37 @@ sub detect_screensaver {
 
   GNOME:
     {
-        last unless Proc::Find::proc_exists(name => "gnome-screensaver");
-        return "gnome";
+        log_trace "Checking whether gnome-screensaver process exists ...";
+        unless (Proc::Find::proc_exists(name => "gnome-screensaver")) {
+            log_trace "gnome-screensaver process doesn't exist";
+            last;
+        }
+        log_trace "gnome-screensaver process exists";
+        log_trace "Concluding screensaver is gnome (<= 3.6)";
+        return "gnome"; # <= 3.6
     }
 
   CINNAMON:
     {
-        last unless Proc::Find::proc_exists(name => "cinnamon-screensaver");
+        log_trace "Checking whether cinnamon-screensaver process exists ...";
+        unless (Proc::Find::proc_exists(name => "cinnamon-screensaver")) {
+            log_trace "cinnamon-screensaver process doesn't exist";
+            last;
+        }
+        log_trace "cinnamon-screensaver process exists";
+        log_trace "Concluding screensaver is cinnamon";
         return "cinnamon";
     }
 
   XSCREENSAVER:
     {
-        last unless Proc::Find::proc_exists(name => "xscreensaver");
+        log_trace "Checking whether xscreensaver process exists ...";
+        unless (Proc::Find::proc_exists(name => "xscreensaver")) {
+            log_trace "xscreensaver process doesn't exist";
+            last;
+        }
+        log_trace "xscreensaver process exists";
+        log_trace "Concluding screensaver is xscreensaver";
         return "xscreensaver";
     }
 
@@ -271,6 +301,13 @@ $SPEC{enable_screensaver} = {
 };
 sub enable_screensaver {
     my %args = @_;
+    my $screensaver = $args{screensaver} // detect_screensaver();
+
+    if ($screensaver eq 'gnome') {
+        system "gsettings", "set", "org.gnome.desktop.lockdown", "disable-lock-screen", "false";
+        if ($?) { return [500, "Failed"] } else { return [200, "OK"] }
+    }
+
     [501, "Not yet implemented"];
 }
 
@@ -283,6 +320,13 @@ $SPEC{disable_screensaver} = {
 };
 sub disable_screensaver {
     my %args = @_;
+    my $screensaver = $args{screensaver} // detect_screensaver();
+
+    if ($screensaver eq 'gnome') {
+        system "gsettings", "set", "org.gnome.desktop.lockdown", "disable-lock-screen", "true";
+        if ($?) { return [500, "Failed"] } else { return [200, "OK"] }
+    }
+
     [501, "Not yet implemented"];
 }
 
@@ -481,7 +525,7 @@ Screensaver::Any - Common interface to screensaver/screenlocker functions
 
 =head1 VERSION
 
-This document describes version 0.002 of Screensaver::Any (from Perl distribution Screensaver-Any), released on 2017-03-31.
+This document describes version 0.003 of Screensaver::Any (from Perl distribution Screensaver-Any), released on 2018-02-28.
 
 =head1 DESCRIPTION
 
@@ -492,6 +536,15 @@ Supported screensavers: KDE Plasma's kscreenlocker (C<kde>), GNOME screensaver
 (C<gnome>), Cinnamon screensaver (C<cinnamon>), and C<xscreensaver>. Support for
 more screensavers, e.g. Windows is more than welcome.
 
+=head1 NOTES
+
+In GNOME 3.8 and later, C<gnome-screensaver> command has been removed (one of
+the reasons is consideration of the eventual move to Wayland). Locking/unlocking
+screen can be done if you install C<gnome-screensaver> separately, or use other
+screensaver like C<xscreensaver>, or use C<gdm> (in which case you can use a
+command like C<< dbus-send --type=method_call --dest=org.gnome.ScreenSaver
+/org/gnome/ScreenSaver org.gnome.ScreenSaver.Lock >>).
+
 =head1 FUNCTIONS
 
 
@@ -863,7 +916,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by perlancar@cpan.org.
+This software is copyright (c) 2018, 2017 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
